use std::str::FromStr;

use i18n::{message, message_ids::MessageId};
use validator::Validate;

use crate::model::response::{ResData, ResErr, ResResult};

/// 验证请求参数
pub fn validate_params(params: &impl Validate, local: &str) -> ResResult<ResData<()>> {
    let validate_err = params.validate();
    match validate_err {
        Ok(_) => Result::Ok(ResData::none()),
        Err(err) => {
            let mut errors = vec![];
            for (_, err) in err.errors() {
                match err {
                    validator::ValidationErrorsKind::Struct(err) => {
                        for (_field, errs) in err.field_errors() {
                            for e in errs {
                                let msg = e.message.clone().unwrap_or_default();
                                errors.push(message!(local, MessageId::from_str(msg.trim()).unwrap()));
                            }
                        }
                    },
                    validator::ValidationErrorsKind::List(err) => {
                        for (_, err) in err.iter() {
                            for (_field, errs) in err.field_errors() {
                                for e in errs {
                                    let msg = e.message.clone().unwrap_or_default();
                                    errors.push(message!(local, MessageId::from_str(msg.trim()).unwrap()));
                                }
                            }
                        }
                    },
                    validator::ValidationErrorsKind::Field(err) => {
                        err.iter().for_each(|e| {
                            let msg = e.message.clone().unwrap_or_default();
                            errors.push(message!(local, MessageId::from_str(msg.trim()).unwrap()))
                        });
                    },
                };
            }
            Err(ResErr::params(errors.join(";")))
        },
    }
}